@@ -2,19 +2,19 @@
 // There are some TODOs in this file:
 //   - fix instrumentation via external call
 //   - fix inline instrumentation
-//   - implement whitelist feature
+//   - implement instrument list feature
 //   - dont instrument blocks that are uninteresting
 //   - implement neverZero
 //
 
 /*
-   american fuzzy lop - GCC instrumentation pass
+   american fuzzy lop++ - GCC instrumentation pass
    ---------------------------------------------
 
    Written by Austin Seipp <aseipp@pobox.com> with bits from
               Emese Revfy <re.emese@gmail.com>
 
-   Fixed by Heiko Eißfeldt 2019 for AFL++
+   Fixed by Heiko Eißfeldt 2019-2020 for AFL++
 
    GCC integration design is based on the LLVM design, which comes
    from Laszlo Szekeres. Some of the boilerplate code below for
@@ -49,16 +49,16 @@
 
 #define BUILD_INLINE_INST
 
-#include "../config.h"
+#include "../include/config.h"
 #include "../include/debug.h"
 
-/* clear helper AFL types pulls in, which intervene with gcc-plugin geaders from
- * GCC-8 */
+/* clear helper macros AFL types pull in, which intervene with gcc-plugin
+ * headers from GCC-8 */
 #ifdef likely
-#undef likely
+  #undef likely
 #endif
 #ifdef unlikely
-#undef unlikely
+  #undef unlikely
 #endif
 
 #include <stdio.h>
@@ -95,7 +95,7 @@
 static int                    be_quiet = 0;
 static unsigned int           inst_ratio = 100;
 static bool                   inst_ext = true;
-static std::list<std::string> myWhitelist;
+static std::list<std::string> myInstrumentList;
 
 static unsigned int ext_call_instrument(function *fun) {
 
@@ -295,16 +295,16 @@ static unsigned int inline_instrument(function *fun) {
     update_stmt(g);
 
 #if 1
-#if 0
+  #if 0
 		tree addr = build2(ADDR_EXPR, map_type, map_ptr, area_off);
 		g = gimple_build_assign(map_ptr2, MODIFY_EXPR, addr);
 		gimple_seq_add_stmt(&seq, g); // map_ptr2 = map_ptr + area_off
 		update_stmt(g);
-#else
+  #else
     g = gimple_build_assign(map_ptr2, PLUS_EXPR, map_ptr, area_off);
     gimple_seq_add_stmt(&seq, g);  // map_ptr2 = map_ptr + area_off
     update_stmt(g);
-#endif
+  #endif
 
     // gimple_assign <mem_ref, _3, *p_6, NULL, NULL>
     tree tmp1 = create_tmp_var_raw(unsigned_char_type_node, "tmp1");
@@ -414,7 +414,7 @@ class afl_pass : public gimple_opt_pass {
 
   unsigned int execute(function *fun) override {
 
-    if (!myWhitelist.empty()) {
+    if (!myInstrumentList.empty()) {
 
       bool         instrumentBlock = false;
       std::string  instFilename;
@@ -436,8 +436,8 @@ class afl_pass : public gimple_opt_pass {
         /* Continue only if we know where we actually are */
         if (!instFilename.empty()) {
 
-          for (std::list<std::string>::iterator it = myWhitelist.begin();
-               it != myWhitelist.end(); ++it) {
+          for (std::list<std::string>::iterator it = myInstrumentList.begin();
+               it != myInstrumentList.end(); ++it) {
 
             /* We don't check for filename equality here because
              * filenames might actually be full paths. Instead we
@@ -462,13 +462,14 @@ class afl_pass : public gimple_opt_pass {
       }
 
       /* Either we couldn't figure out our location or the location is
-       * not whitelisted, so we skip instrumentation. */
+       * not in the instrument list, so we skip instrumentation. */
       if (!instrumentBlock) {
 
         if (!be_quiet) {
 
           if (!instFilename.empty())
-            SAYF(cYEL "[!] " cBRI "Not in whitelist, skipping %s line %u...\n",
+            SAYF(cYEL "[!] " cBRI
+                      "Not in instrument list, skipping %s line %u...\n",
                  instFilename.c_str(), instLine);
           else
             SAYF(cYEL "[!] " cBRI "No filename information found, skipping it");
@@ -502,7 +503,7 @@ int plugin_is_GPL_compatible = 1;
 
 static struct plugin_info afl_plugin_info = {
 
-    .version = "20191015",
+    .version = "20200519",
     .help = "AFL++ gcc plugin\n",
 
 };
@@ -528,12 +529,13 @@ int plugin_init(struct plugin_name_args *  plugin_info,
 
   if (!plugin_default_version_check(version, &gcc_version)) {
 
-    FATAL(G_("Incompatible gcc/plugin versions!"));
+    FATAL(G_("Incompatible gcc/plugin versions! Expected GCC %d.%d"),
+          GCCPLUGIN_VERSION_MAJOR, GCCPLUGIN_VERSION_MINOR);
 
   }
 
   /* Show a banner */
-  if (isatty(2) && !getenv("AFL_QUIET")) {
+  if ((isatty(2) && !getenv("AFL_QUIET")) || getenv("AFL_DEBUG") != NULL) {
 
     SAYF(G_(cCYA "afl-gcc-pass" VERSION cRST
                  " initially by <aseipp@pobox.com>, maintainer: hexcoder-\n"));
@@ -561,26 +563,32 @@ int plugin_init(struct plugin_name_args *  plugin_info,
 
   }
 
-  char *instWhiteListFilename = getenv("AFL_GCC_WHITELIST");
-  if (instWhiteListFilename) {
+  char *instInstrumentListFilename = getenv("AFL_GCC_INSTRUMENT_FILE");
+  if (!instInstrumentListFilename)
+    instInstrumentListFilename = getenv("AFL_GCC_WHITELIST");
+  if (instInstrumentListFilename) {
 
     std::string   line;
     std::ifstream fileStream;
-    fileStream.open(instWhiteListFilename);
-    if (!fileStream) fatal_error(0, "Unable to open AFL_GCC_WHITELIST");
+    fileStream.open(instInstrumentListFilename);
+    if (!fileStream) PFATAL("Unable to open AFL_GCC_INSTRUMENT_FILE");
     getline(fileStream, line);
     while (fileStream) {
 
-      myWhitelist.push_back(line);
+      myInstrumentList.push_back(line);
       getline(fileStream, line);
 
     }
 
-  } else if (!be_quiet && getenv("AFL_LLVM_WHITELIST"))
+  } else if (!be_quiet && (getenv("AFL_LLVM_WHITELIST") ||
+
+                           getenv("AFL_LLVM_INSTRUMENT_FILE"))) {
 
     SAYF(cYEL "[-] " cRST
-              "AFL_LLVM_WHITELIST environment variable detected - did you mean "
-              "AFL_GCC_WHITELIST?\n");
+              "AFL_LLVM_INSTRUMENT_FILE environment variable detected - did "
+              "you mean AFL_GCC_INSTRUMENT_FILE?\n");
+
+  }
 
   /* Go go gadget */
   register_callback(plugin_info->base_name, PLUGIN_INFO, NULL,
